function [naf,tau,xi]=ambifunb(x,tau,N,ptrace,opt_plot);
//  Narrow-band ambiguity function.
// Calling Sequence
//	[NAF,TAU,XI]=ambifunb(X) 
//	[NAF,TAU,XI]=ambifunb(X,TAU)
//	[NAF,TAU,XI]=ambifunb(X,TAU,N)  
//	[NAF,TAU,XI]=ambifunb(X,TAU,N,TRACE)
//	[NAF,TAU,XI]=ambifunb(...,'plot')  
// Parameters
//	X     : signal if auto-AF, or [X1,X2] if cross-AF (length(X)=Nx).
//	TAU   : vector of lag values     (default is -Nx/2:Nx/2).
//	N     : number of frequency bins (default is length(X)).
//	TRACE : if 1,              (default is 0)  the progression of the algorithm is shown.
//	'plot'      :  if input contains the string 'plot', the output values will be plotted
//	NAF   : doppler-lag representation, with the doppler bins stored in the rows and the time-lags stored in the columns. When called without output arguments, AMBIFUNB displays  the squared modulus of the ambiguity function by means of contour.
//	XI    : vector of doppler values.
// Description
//      ambifunb computes the narrow-band ambiguity function of a signal X, or the cross-ambiguity 
//	function between two signals.
//  Examples
//       sig=anabpsk(256,8);
//       ambifunb(sig,'plot'); 
//  See also
//    ambifuwb
// Authors
//   H. Nahrstaedt - Aug 2010
//   O. Lemoine
//   F. Auger - August 1995.

//	Copyright (c) 1996 by CNRS (France).
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  [nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least one parameter required');
end;
in_par=['x','tau','N','ptrace','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

[xrow,xcol] = size(x);
if (xcol==0)|(xcol>2),
 error('X must have one or two columns');
end;

if (nargin == 1),
 if modulo(xrow,2)==0, tau=(-xrow/2+1):(xrow/2-1); 
 else tau=(-(xrow-1)/2):((xrow+1)/2-1); end
 N=xrow; ptrace=0;
elseif (nargin == 2),
 N=xrow ; ptrace=0;
elseif (nargin == 3),
 ptrace=0;
end;

[taurow,taucol] = size(tau);

if (taurow~=1),
 error('TAU must only have one row'); 
elseif (N<0),
 error('N must be greater than zero');
end;

naf=zeros(N,taucol); 
if ptrace, disp('Narrow-band ambiguity function'); end;
for icol=1:taucol,
 if ptrace, disprog(icol,taucol,10); end;
 taui=tau(icol);
 t=(1+abs(taui)):(xrow-abs(taui));
 naf(t,icol)=x(t+taui,1).* conj(x(t-taui,xcol));
end;

naf=tftb_fft(naf);
naf=naf([(N+modulo(N,2))/2+1:N 1:(N+modulo(N,2))/2],:);

xi=(-(N-modulo(N,2))/2:(N+modulo(N,2))/2-1)/N;

if (plotting),
 xset("fpf"," ");
 contour2d(2*tau,xi,(abs(naf).^2)',16); 

 xgrid

 xlabel('Delay'); ylabel('Doppler')
 title('Narrow-band ambiguity function');
end;
endfunction
