function tifd=friedman(tfr,rhat,t,method,ptrace,opt_plot);
//FRIEDMAN Instantaneous frequency density.
// Calling Sequence
//	TIFD = friedman(TFR,HAT 
//	TIFD = friedman(TFR,HAT,T 
//	TIFD = friedman(TFR,HAT,T,METHOD) 
//	TIFD = friedman(TFR,HAT,T,METHOD,TRACE) 
//	TIFD = friedman(...,'plot') 
//  Parameters
//	TFR   : time-frequency representation, (N,M) matrix.
//	HAT   : complex matrix of the reassignment vectors.
//	T     : the time instant(s)	(default : (1:M)).
//	METHOD: chosen representation	(default : 'tfrrsp').  
//      'plot':	when called with the additional string 'plot',  friedman runs tfrqview. and TIFD will be plotted
//	TRACE : if nonzero, the progression of the algorithm is shown	(default : 0).
//	TIFD  : time instantaneous-frequency density.
// Description
//      computes the	time-instantaneous frequency density (defined by Friedman [1])
//	of a reassigned time-frequency representation.
//
//	WARNING : TIFD is not an energy distribution, but an estimated 
//	-------        probability distribution !
//   Examples
//       sig=fmlin(128,0.1,0.4); h=tftb_window(47,'Kaiser');
//       t=1:2:127; [tfr,rtfr,hat]=tfrrpwv(sig,t,128,h);
//       friedman(tfr,hat,t,'tfrrpwv',1,'plot'); 
//    See also
//        ridges
//   Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, August 1994, Decembre 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//	[1] : D. H. Friedman, "Instantaneous Frequency vs Time : An
//	      Interpretation of the Phase Structure of Speech", Proc. IEEE
//	      ICASSP, pp. 29.10.1-4, Tampa, 1985.	
  [nargout,nargin]=argn(0);
if (nargin<3),
 error('At least 2 parameters required'); 
end;

in_par=['tfr','rhat','t','method','ptrace','opt_plot'];
in_par_min=2;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;


[tfrrow,tfrcol]=size(tfr);
[hatrow,hatcol]=size(rhat);

if (nargin==2),
 t=1:tfrcol; method='tfrrsp'; ptrace=0;
elseif (nargin==3),
 method='tfrrsp'; ptrace=0;
elseif (nargin==4),
 ptrace=0;
end;

[trow,tcol] = size(t);
if (trow~=1),
 error('T must only have one row'); 
elseif (tfrrow~=hatrow)|(tfrcol~=hatcol),
 error('tfr and hat must have the same size');
end;

tifd=zeros(tfrrow,tfrcol);
bins=0.5+(0:tfrrow-1);
threshold=sum(tfr)*0.5/(tfrrow*tfrcol);

if ptrace, printf ('\nFriedman distribution: \n'); end;

for j=1:tfrcol,
 if ptrace, disprog(j,tfrcol,10); end;
 indices=find(tfr(:,j)>threshold);
 if (length(indices)>=1),
  [occurences,trash]=hist(real(rhat(indices,j)),bins);
  tifd(:,j)=occurences;
 end;
end; 
tifd=tifd/sum(tifd);

method=convstr(method,"u");
if plotting,
 tfrqview(tifd,[],t,method);
end
endfunction



function [h,t] = hist(x,n)

// hist - compute and display histogram
//
//  [h,t] = hist(x,n);
//
//  if h is ommitted, then display the histogram, 
//  otherwise, compute it.
//
//  Copyright (c) 2008 Gabriel Peyre


if argn(2)<2
    n = 100;
end


x = x(:);
if length(n)>1
    t = n;
    n = length(n);
    // closest point histograms
    h = zeros(n,1);
    for i=1:length(x)
        [tmp,j] = min( abs(x(i)-t(:)), 'r' );
        h(j) = h(j)+1;
    end
else
    // equispaced histograms
    a = min(x); b = max(x);
    tau = (b-a)/n;
    a1 = a+tau/2; b1 = b-tau/2;
    t = a1:tau:b1;
    x1 = (x-a1)/(b1-a1)*(n-1)+1;
    x1 = round(x1);
    h = zeros(n,1);
    for i=1:n
        h(i) = sum(x1==i);
    end
end
h = h/sum(h);



endfunction
    




