function [pointst,pointsf]=ridges(tfr,mhat,t,method,ptrace,opt_plot);
//  Extraction of ridges.
// Calling Sequence
//	[POINTST,POINTSF]=ridges(TFR,HAT) 
//	[POINTST,POINTSF]=ridges(TFR,HAT,T) 
//	[POINTST,POINTSF]=ridges(TFR,HAT,T,METHOD) 
//	[POINTST,POINTSF]=ridges(TFR,HAT,T,METHOD,TRACE) 
//	[POINTST,POINTSF]=ridges(...,'plot') 
// Parameters
//	TFR    : time-frequency representation
//	HAT    : complex matrix of the reassignment vectors.
//	T      : the time instant(s).
//	METHOD : the chosen representation (default: 'tfrrsp'). 
//	TRACE  : if nonzero, the progression of the algorithm is shown   (default : 0).
//      'plot':	when called with the additional string 'plot', the output values will be plotted
//	POINTST,POINTSF are two vectors for the time and frequency coordinates of the stationary points of the reassignment. Therefore, PLOT(POINTST,POINTSF,'.') shows the squeleton of the representation.
//  Description
//      ridges xtracts the ridges of a time-frequency distribution. These ridges are some
//	particular sets of curves deduced from the stationary points of
//	their  reassignment operators.
//  Examples
//	 sig=fmlin(128,0.1,0.4); g=tftb_window(21,'kaiser'); 
//	 h=tftb_window(47,'Kaiser'); t=1:2:127; 
//	 figure(1), [tfr,rtfr,hat]=tfrrspwv(sig,t,128,g,h); 
//	 ridges(tfr,hat,t,'tfrrspwv',1,'plot');
//	 figure(2), [tfr,rtfr,hat]=  tfrrsp(sig,t,128,h);   
//	 ridges(tfr,hat,t,'tfrrsp',1,'plot');
//  See also 
//   friedman
//   Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, August 1994, December 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 [nargout,nargin]=argn(0);
if (nargin<2),
 error('at least 2 parameters required'); 
end;

plotting=0;
if (nargin == 3)
  if (type(t)==10)
    nargin=nargin-1;
    if convstr(t)=="plot",   plotting=1; end; clear t;
  end
elseif (nargin == 4)
  if  convstr(method)=="plot"
    nargin=nargin-1;
      plotting=1;
       clear method;
  end
elseif (nargin == 5)
  if (type(ptrace)==10)
    nargin=nargin-1;
    if convstr(t)=="plot",   plotting=1; end; clear ptrace;
  end
elseif nargin==6
  nargin=nargin-1;
  if convstr(opt_plot)=="plot",   plotting=1; end; clear opt_plot;
end;

[tfrrow,tfrcol]=size(tfr);
[hatrow,hatcol]=size(mhat);

if (nargin==2),
 t=1:tfrcol; method='tfrrsp'; ptrace=0;
elseif (nargin==3),
 method='tfrrsp'; ptrace=0; 
elseif (nargin==4),
 ptrace=0; 
end;

[trow,tcol] = size(t);
if (trow~=1),
 error('T must only have one row'); 
elseif (tfrrow~=hatrow)|(tfrcol~=hatcol),
 error('TFR and HAT must have the same size');
end;

Nf=tfrrow; frequencies=(1:Nf)'; 
threshold=sum(tfr)*0.5/(tfrrow*tfrcol);

pointst=[]; pointsf=[];

if ptrace, printf ('\nRidge extraction: \n'); end;

method=convstr(method,"u")
if (method=='TFRRPWV') | (method=='TFRRPMH'),
 for icol=1:tfrcol, ti=t(icol); 
  if ptrace, disprog(icol,tfrcol,10); end;
  indices=find((tfr(:,icol)>threshold)&(mhat(:,icol)-frequencies==0)); 
  nbindices=length(indices);
  if (nbindices>0), 
   pointst=[pointst;ones(nbindices,1)*ti];
   pointsf=[pointsf;indices/(2.0*Nf)]; 
  end;
 end;
elseif (method=='TFRRSPWV'),
 for icol=1:tfrcol, ti=t(icol); 
  if ptrace, disprog(icol,tfrcol,10); end;
  indices=find((real(mhat(:,icol))-frequencies==0)&...
               (imag(mhat(:,icol))-icol==0)&...
               (tfr(:,icol)>threshold)); 
  nbindices=length(indices);
  if (nbindices>0), 
   pointst=[pointst;ones(nbindices,1)*ti];
   pointsf=[pointsf;indices/(2.0*Nf)]; 
  end;
 end;
elseif (method=='TFRRSP')|strcmp(method,'TYPE1')
 for icol=1:tfrcol, ti=t(icol); 
  if ptrace, disprog(icol,tfrcol,10); end;
  indices=find((real(mhat(:,icol))-frequencies==0)&...
               (imag(mhat(:,icol))-icol==0)&...
               (tfr(:,icol)>threshold))'; 
  nbindices=length(indices);
  if (nbindices>0), 
   pointst=[pointst;ones(nbindices,1)*ti];
   pointsf=[pointsf;indices/Nf]; 
  end;
 end;
else 
 error('unknown representation');
end;

if (plotting),
 clf;
 plot(pointst,pointsf,'.')  
a=gca();a.data_bounds=[min(t) max(t) 0 0.5];
end;
endfunction