function [tfr,t,f] = tfrsp(x,t,N,h,ttrace);
//TFRSP	Spectrogram time-frequency distribution.
//	[TFR,T,F]=TFRSP(X,T,N,H,TTRACE) computes the Spectrogram 
//	distribution of a discrete-time signal X. 
// 
//	X     : signal.
//	T     : time instant(s)          (default : 1:length(X)).
//	N     : number of frequency bins (default : length(X)).
//	H     : analysis window, H being normalized so as to
//	        be  of unit energy.      (default : Hamming(N/4)). 
//	TTRACE : if nonzero, the progression of the algorithm is shown
//	                                 (default : 0).
//	TFR   : time-frequency representation. When called without 
//	        output arguments, TFRSP runs TFRQVIEW.
//	F     : vector of normalized frequencies.
//
//	Example :
//	 sig=fmlin(128,0.1,0.4);
//	 h=tftb_window(17,'Kaiser'); tfrsp(sig,1:128,64,h,1);
//
//        [tfr,t,freq]=tfrsp(sig,1:128,64,h,1); plot(fftshift(freq),fftshift(tfr(:,100)))
//
//	See also all the time-frequency representations listed in
//	 the file CONTENTS (TFR*)

//	F. Auger, May-August 1994, July 1995.
//	Copyright (c) 1996 by CNRS (France).
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
[nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;
[xrow,xcol] = size(x);
if (nargin < 1),
 error('At least 1 parameter is required');
elseif (nargin <= 2),
 N=xrow;
end;

hlength=floor(N/4);
hlength=hlength+1-modulo(hlength,2);

if (nargin == 1),
 t=1:xrow; h = tftb_window(hlength); ttrace=0;
elseif (nargin == 2) | (nargin == 3),
 h = tftb_window(hlength); ttrace=0;
elseif (nargin == 4),
 ttrace = 0;
end;

if (N<0),
 error('N must be greater than zero');
end;
[trow,tcol] = size(t);
if (xcol==0) | (xcol>2),
 error('X must have one or two columns');
elseif (trow~=1),
 error('T must only have one row'); 
elseif (2^nextpow2(N)~=N),
 fprintf('For a faster computation, N should be a power of two\n');
end; 

[hrow,hcol]=size(h); Lh=(hrow-1)/2;
if ((hcol~=1)|(modulo(hrow,2)==0)),
 error('H must be a smoothing window with odd length');
end;

tfr= zeros (N,tcol) ;  
//if ttrace, disp('Spectrogram'); end;
for icol=1:tcol,
 ti= t(icol); tau=-min([round(N/2)-1,Lh,ti-1]):min([round(N/2)-1,Lh,xrow-ti]);
 indices= modulo(N+tau,N)+1; 
 //if ttrace, disprog(icol,tcol,10); end;
 tfr(indices,icol)=x(ti+tau).*conj(h(Lh+1+tau))/norm(h(Lh+1+tau));
end;
tfr=abs(mtlb_fft(tfr)).^2; 
//if ttrace, fprintf('\n'); end;

//if (nargout==0),
// tfrqview(tfr,x,t,'tfrsp',h);
//elseif (nargout==3),
 if modulo(N,2)==0, 
  f=[0:N/2-1 -N/2:-1]'/N;
 else
  f=[0:(N-1)/2 -(N-1)/2:-1]'/N;  
 end;
//end;

endfunction
