function [tfr,t,f] = tfrmmce(x,h,t,N,ptrace,opt_plot);
// Minimum mean cross-entropy combination of spectrograms.
// Calling Sequence
//	[TFR,T,F]=tfrmmce(X,H)
//	[TFR,T,F]=tfrmmce(X,H,T)
//	[TFR,T,F]=tfrmmce(X,H,T,N)
//	[TFR,T,F]=tfrmmce(X,H,T,N,TRACE)
//	[TFR,T,F]=tfrmmce(...,'plot')
//  Parameters
//	X     : signal.
//	H     : frequency smoothing windows, the H(:,i) being normalized   so as to be of unit energy. 
//	T     : time instant(s)          (default : 1:length(X)).
//	N     : number of frequency bins (default : length(X))
//	TRACE : if nonzero, the progression of the algorithm is shown         (default : 0).
//      'plot':	if one input parameter is 'plot',  tfrmh runs tfrqview. and TFR will be plotted
//	TFR   : time-frequency representation. 
//	F     : vector of normalized frequencies.
//  Description
//       tfrmmce  computes the minimum mean cross-entropy combination of spectrograms using as 
//	windows the columns of the matrix H.
//  Examples
//       sig=fmlin(128,0.1,0.4); h=zeros(19,3);
//       h(10+(-5:5),1)=tftb_window(11); h(10+(-7:7),2)=tftb_window(15);  
//       h(10+(-9:9),3)=tftb_window(19); tfrmmce(sig,h,'plot');
//     Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, August 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 [nargout,nargin]=argn(0);

in_par=['x','h','t','N','ptrace','opt_plot'];
in_par_min=2;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

[xrow,xcol] = size(x);
if (nargin < 2),
 error('At least 2 parameters are required');
elseif (nargin==2),
 t=1:xrow; N=xrow; ptrace=0;
elseif (nargin==3),
 N=xrow; ptrace=0;
elseif (nargin==4),
 ptrace=0;
end;
[trow,tcol] = size(t);

if (xcol~=1),
 error('X must have only one column');
elseif (trow~=1),
 error('T must only have one row');
elseif (N<=0),
 error('N must be greater than zero'); 
elseif (2^nextpow2(N)~=N & nargin==5),
 printf('For a faster computation, N should be a power of two\n');
end; 

[hrow,hcol]=size(h); Lh=(hrow-1)/2; 
if (modulo(hrow,2)==0),
 error('H must have an odd number of lines');
elseif hcol==1, 
 error('H must have at least two columns');
end;
h=h*diag(1.0 ./ sqrt(mtlb_sum(abs(h).^2)));

tfr= zeros (N,tcol);
slides= zeros(N,hcol);
if ptrace, disp('MMCE Spectrogram'); end;

for icol=1:tcol,
 ti= t(icol); tau=-min([round(N/2)-1,Lh,ti-1]):min([round(N/2)-1,Lh,xrow-ti]);
 indices= modulo(N+tau,N)+1; slides= zeros(N,hcol);
 if ptrace, disprog(icol,tcol,10); end;
 for ih=1:hcol,
  slides(indices,ih)=x(ti+tau).*conj(h(Lh+1+tau,ih));
 end;
 slides=abs(tftb_fft(slides)).^2;

 tfr(:,icol)=mtlb_prod(slides')' .^(1/hcol);
end;

tfr=tfr*(sum(abs(x(t)).^2)/sum(sum(tfr)));
if ptrace, printf('\n'); end;

if (plotting),
 tfrqview(tfr,x,t,'tfrmmce',h);
end
if (nargout==3),
 //f=fftshift((-(N/2):(N/2)-1)/N)';
  f=(0:N-1)/N;	
end;

endfunction


