function [tfr,t,f] = tfrparam(x,t,N,p,L,ptrace,Rxtype,method,q,opt_plot);
//parametric time-frequency representation.
// Calling Sequence
// [tfr,t,f]=tfrparam(x,t,N,p,L,ptrace,Rxtype,method,q) 
// Parameters
// X      : signal
// T      : time instant(s)          (default : 1:length(X)).
// N      : number of frequency bins (default : max(256,length(X)) ).
// p      : last autocorrelation lag (default : 2 ).
// L      : length of the window around the analyzed time sample. Must be odd.         (default : max(51,round(xrow/10)) ).
// Rxtype : choice of the correlation matrix algorithm (default : 'fbhermitian')   possible values are 'hermitian', 'fbhermitian', 'burg' or 'fbburg'
// method : can be either 'ar', 'periodogram', 'capon', 'capnorm', 'lagunas',   or 'genlag'.
// q      : parameter for the generalized Lagunas method.
//      'plot':	if one input parameter is 'plot',  tfrparam runs tfrqview. and TFR will be plotted
// Examples
//          x=fmconst(256,0.1)+fmlin(256,0.15,0.30)+fmlin(256,0.2,0.45);
//          scf(1); tfrparam(x,1:256,256,3,21,1,'fbhermitian','ar','plot');
//          scf(2); tfrparam(x,1:256,256,15,61,1,'fbhermitian','periodogram','plot');
//          scf(3); tfrparam(x,1:256,256,3,21,1,'fbhermitian','capon','plot');
//          scf(4); tfrparam(x,1:256,256,3,21,1,'fbhermitian','lagunas','plot');
//     Authors
//      H. Nahrstaedt - Aug 2010
// F. Auger, july, november 1998, april 99.


//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 [nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;

in_par=['x','t','N','p','L','h','ptrace','Rxtype','method','q','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

[xrow,xcol] = size(x);
if (nargin==1),
 t=1:xrow; N=max(256,xrow); p=2; L=max(51,round(xrow/10)); ptrace=0; Rxtype='fbhermitian'; method='ar';
elseif (nargin==2),
 N=max(256,xrow); p=2; L=max(51,round(xrow/10)); ptrace=0; Rxtype='fbhermitian'; method='ar';
elseif (nargin==3),
 p=2; L=max(51,round(xrow/10)); ptrace=0; Rxtype='fbhermitian'; method='ar';
elseif (nargin==4),
 L=max(51,round(xrow/10)); ptrace=0; Rxtype='fbhermitian'; method='ar';
elseif (nargin==5),
 ptrace=0; Rxtype='fbhermitian'; method='ar';
elseif (nargin==6),
 Rxtype='fbhermitian'; method='ar';
elseif (nargin==7),
 method='ar';
end;
 
[trow,tcol] = size(t);

if xcol>1,
 error('x must be a column vector');
elseif p<1,
 error('p must be greater than 0');
end;

if (modulo(L,2)==0), // L must be odd
 L=L+1; printf('L corrected to L+1\n');
end;
Lhalf=(L-1)/2;

tfr= zeros (N,tcol) ;  
if ptrace, disp('Tfrparam'); end;
for icol=1:tcol,
 ti= t(icol);
 if ti-Lhalf >= 1,    timin=ti-Lhalf; else timin=1; end;
 if ti+Lhalf <= xrow, timax=ti+Lhalf; else timax=xrow; end;
 if ptrace, disprog(icol,tcol,10); end;
 Realp=min (p,timax-timin); 
 Rx=correlmx(x(timin:timax),Realp,Rxtype);
 if (convstr(method,"u")=='GENLAG'),
  [tfr(:,icol),f]=parafrep(Rx,N,method,q);
 else
  [tfr(:,icol),f]=parafrep(Rx,N,method);
 end
end;
if ptrace, printf('\n'); end;

if (plotting),
 grayplot(t,f,10.0*log10(tfr)); //axis('xy'); 
 title('tfrparam '+method+...
        ', p='+string(p)+...
        ', L='+string(L)+', '+...
        Rxtype+' autocorrelation matrix');
end;

endfunction

