function [tfr,rtfr,rhat] = tfrrpmh(x,t,N,h,ptrace,opt_plot);
// Reassigned pseudo Margenau-Hill time-frequency distribution.
// Calling Sequence
//	[TFR,RTFR,HAT] = tfrrpmh(X) 
//	[TFR,RTFR,HAT] = tfrrpmh(X,T) 
//	[TFR,RTFR,HAT] = tfrrpmh(X,T,N) 
//	[TFR,RTFR,HAT] = tfrrpmh(X,T,N,H) 
//	[TFR,RTFR,HAT] = tfrrpmh(X,T,N,H,TRACE) 
//	[TFR,RTFR,HAT] = tfrrpmh(...,'plot') 
// Parameters
//	X     : analysed signal,
//	T     : time instant(s)          (default : 1:length(X)).
//	N     : number of frequency bins (default : length(X)).
//	H     : frequency smoothing window, H(0) being forced to 1      (default : Hamming(N/4)).
//	TRACE : if nonzero, the progression of the algorithm is shown   (default : 0).
//      'plot':	if one input parameter is 'plot',  tfrrpmh runs tfrqview. and TFR will be plotted
//	TFR,  : time-frequency representation and its reassigned
//	RTFR    version. 
//	HAT   : Complex matrix of the reassignment vectors.
// Description
//	tfrrpmh computes the pseudo Margenau-Hill distribution
//	and its reassigned version.
//  Examples
//       sig=fmlin(128,0.1,0.4); t=1:2:128; 
//       h=tftb_window(17,'Kaiser'); tfrrpmh(sig,t,64,h,1,'plot');
//     Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, May-July 1994, July 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 [nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;

in_par=['x','t','N','h','ptrace','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;


[xrow,xcol] = size(x);

if (nargin <= 2),
 N=xrow;
end;

hlength=floor(N/4);
if (modulo(hlength,2)==0),
 hlength=hlength+1;
end;

if (nargin == 1),
 t=1:xrow; h = tftb_window(hlength); ptrace=0;
elseif (nargin == 2)|(nargin == 3),
 h = tftb_window(hlength); ptrace=0;
elseif (nargin == 4),
 ptrace = 0;
end;

if (N<0),
 error('N must be greater than zero');
end;
[trow,tcol] = size(t);
if (xcol~=1),
 error('X must only have one column');
elseif (trow~=1),
 error('T must only have one row'); 
elseif (2^nextpow2(N)~=N & nargin==5),
 printf('For a faster computation, N should be a power of two\n');
end; 

[hrow,hcol]=size(h); Lh=(hrow-1)/2; h=h/h(Lh+1);
if (hcol~=1)|(modulo(hrow,2)==0),
 error('H must be a smoothing window with odd length');
end;

if (tcol==1),
 Dt=1; 
else
 Deltat=t(2:tcol)-t(1:tcol-1); 
 Mini=min(Deltat); Maxi=max(Deltat);
 if (Mini~=Maxi),
  error('The time instants must be regularly sampled.');
 else
  Dt=Mini;
 end;
 clear Deltat Mini Maxi;
end;

tfr= zeros(N,tcol); tf2= zeros(N,tcol);
if ptrace, disp('Pseudo Margenau-Hill distribution'); end;
Dh=dwindow(h);
for icol=1:tcol,
 ti= t(icol); tau=-min([round(N/2)-1,Lh,xrow-ti]):min([round(N/2)-1,Lh,ti-1]);
 indices= modulo(N+tau,N)+1;
 if ptrace, disprog(icol,tcol,10); end;
 tfr(indices,icol)= h(Lh+1+tau).*x(ti).*conj(x(ti-tau));
 tf2(indices,icol)=Dh(Lh+1+tau).*x(ti).*conj(x(ti-tau));
end ;

tfr= tftb_fft(tfr); tf2=tftb_fft(tf2);
tfr=tfr(:); tf2=tf2(:);

avoid_warn=find(tfr~=0); 
tf2(avoid_warn)= round(N*tf2(avoid_warn)./tfr(avoid_warn)/(2.0*%pi)); 
tfr=real(tfr); tf2=imag(tf2);
if ptrace, printf ('\nreassignment: \n'); end;
tfr=matrix(tfr,N,tcol);
tf2=matrix(tf2,N,tcol);

rtfr= zeros(N,tcol); 
Ex=mean(abs(x(min(t):max(t))).^2); Threshold=1.0e-6*Ex;
for icol=1:tcol,
 if ptrace, disprog(icol,tcol,10); end;
 for jcol=1:N,
  if abs(tfr(jcol,icol))>Threshold,
   jcolhat= real(jcol - tf2(jcol,icol));
   jcolhat=modulo(modulo(jcolhat-1,N)+N,N)+1;
   rtfr(jcolhat,icol)=rtfr(jcolhat,icol) + tfr(jcol,icol) ;
   tf2(jcol,icol)=jcolhat;
  else 
   tf2(jcol,icol)=%inf;
   rtfr(jcol,icol)=rtfr(jcol,icol) + tfr(jcol,icol) ;
  end;
 end;
end;

if ptrace, printf('\n'); end;
if (plotting),
 TFTBcontinue=1;
 while (TFTBcontinue==1),
  choice=x_choose_modeless (['pseudo Margenau-Hill distribution',...
               'reassigned pseudo Margenau-Hill distribution'],'Choose the representation:','stop');
  if (choice==0), TFTBcontinue=0;
  elseif (choice==1), 
   tfrqview(tfr,x,t,'tfrpmh',h);
  elseif (choice==2),
   tfrqview(rtfr,x,t,'tfrrpmh',h);
  end;
 end;
end
if (nargout>2),
 rhat=tf2;
end;
endfunction
