function [tfr,rtfr,rhat] = tfrrsp(x,t,N,h,ptrace,opt_plot);
// Reassigned Spectrogram.
// Calling Sequence
//	[TFR,RTFR,HAT] = TFRRSP(X) 
//	[TFR,RTFR,HAT] = TFRRSP(X,T) 
//	[TFR,RTFR,HAT] = TFRRSP(X,T,N) 
//	[TFR,RTFR,HAT] = TFRRSP(X,T,N,H) 
//	[TFR,RTFR,HAT] = TFRRSP(X,T,N,H,TRACE) 
//	[TFR,RTFR,HAT] = TFRRSP(...,'plot') 
//  Parameters
//	X     : analysed signal.
//	T     : the time instant(s)      (default : 1:length(X)).
//	N     : number of frequency bins (default : length(X)).
//	H     : frequency smoothing window, H(0) being forced to 1     (default : Hamming(N/4)).
//	TRACE : if nonzero, the progression of the algorithm is shown       (default : 0).
//      'plot':	if one input parameter is 'plot',  tfrrsp runs tfrqview. and TFR will be plotted
//	TFR,  : time-frequency representation and its reassigned
//	RTFR    version. 
//	HAT   : Complex matrix of the reassignment vectors.
// Description
//     tfrrsp computes the spectrogram and its reassigned version.
// Examples
//       sig=fmlin(128,0.1,0.4); t=1:2:128;
//       h=tftb_window(17,'Kaiser'); tfrrsp(sig,t,64,h,1,'plot');
//     Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, May-July 1994, July 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 [nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;

in_par=['x','t','N','h','ptrace','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

[xrow,xcol] = size(x);
if (nargin <= 2),
 N=xrow;
end;

hlength=floor(N/4);
hlength=hlength+1-modulo(hlength,2);

if (nargin == 1),
 t=1:xrow; h = tftb_window(hlength); ptrace=0;
elseif (nargin == 2)|(nargin == 3),
 h = tftb_window(hlength); ptrace=0;
elseif (nargin == 4),
 ptrace = 0;
end;

if (N<0),
 error('N must be greater than zero');
end;
[trow,tcol] = size(t);
if (xcol~=1),
 error('X must have only one column');
elseif (trow~=1),
 error('T must only have one row'); 
elseif (2^nextpow2(N)~=N),
 printf('For a faster computation, N should be a power of two\n');
end; 

[hrow,hcol]=size(h); Lh=(hrow-1)/2; 
if (hcol~=1)|(modulo(hrow,2)==0),
 error('H must be a smoothing window with odd length');
end;

if (tcol==1),
 Dt=1; 
else
 Deltat=t(2:tcol)-t(1:tcol-1); 
 Mini=min(Deltat); Maxi=max(Deltat);
 if (Mini~=Maxi),
  error('The time instants must be regularly sampled.');
 else
  Dt=Mini;
 end;
 clear Deltat Mini Maxi;
end;

tfr= zeros(N,tcol); tf2= zeros(N,tcol); tf3= zeros (N,tcol);
if ptrace, disp('Spectrogram'); end;
Th=h.*[-Lh:Lh]'; Dh=dwindow(h);
for icol=1:tcol,
 ti= t(icol); 
 tau=-min([round(N/2)-1,Lh,ti-1]):min([round(N/2)-1,Lh,xrow-ti]);
 indices= modulo(N+tau,N)+1;
 if ptrace, disprog(icol,tcol,10); end;
 norm_h=norm(h(Lh+1+tau));
 tfr(indices,icol)=x(ti+tau).*conj( h(Lh+1+tau)) /norm_h;
 tf2(indices,icol)=x(ti+tau).*conj(Th(Lh+1+tau)) /norm_h;
 tf3(indices,icol)=x(ti+tau).*conj(Dh(Lh+1+tau)) /norm_h;
end ;
tfr=tftb_fft(tfr);
// tf2=round(real(mtlb_fft(tf2)./tfr/Dt));
// tf3=round(imag(N*mtlb_fft(tf3)./tfr/(2.0*pi)));
tf2=tftb_fft(tf2);
tf3=tftb_fft(tf3);


tfr=tfr(:);tf2=tf2(:);tf3=tf3(:);
avoid_warn=find(tfr~=0);
tf2(avoid_warn)=round(real(  tf2(avoid_warn)./tfr(avoid_warn)/Dt));
tf3(avoid_warn)=round(imag(N*tf3(avoid_warn)./tfr(avoid_warn)/(2.0*%pi)));
tfr=abs(tfr).^2;
if ptrace, printf ('\nreassignment: \n'); end;
tfr=matrix(tfr,N,tcol);
tf2=matrix(tf2,N,tcol);
tf3=matrix(tf3,N,tcol);

rtfr= zeros(N,tcol); 
Ex=mean(abs(x(min(t):max(t))).^2); Threshold=1.0e-6*Ex;
for icol=1:tcol,
 if ptrace, disprog(icol,tcol,10); end;
 for jcol=1:N,
  if abs(tfr(jcol,icol))>Threshold,
   icolhat= icol + tf2(jcol,icol);
   icolhat=min(max(real(icolhat),1),tcol);
   jcolhat= real(jcol - tf3(jcol,icol));
   jcolhat=real(modulo(modulo(jcolhat-1,N)+N,N)+1);
   rtfr(jcolhat,icolhat)=rtfr(jcolhat,icolhat) + tfr(jcol,icol) ;
   tf2(jcol,icol)=jcolhat + %i * icolhat;
  else
   tf2(jcol,icol)=%inf*(1+%i);
   rtfr(jcol,icol)=rtfr(jcol,icol) + tfr(jcol,icol) ;
  end;
 end;
end;

if ptrace, printf('\n'); end;
clear tf3;
if (plotting),
 TFTBcontinue=1;
 while (TFTBcontinue==1),
  choice=x_choose_modeless (['spectrogram',...
               'reassigned spectrogram'],'Choose the representation:','stop');
  if (choice==0), TFTBcontinue=0;
  elseif (choice==1), 
   tfrqview(tfr,x,t,'tfrsp',h);
  elseif (choice==2),
   tfrqview(rtfr,x,t,'tfrrsp',h);
  end;
 end;
end
if (nargout>2),
 rhat=tf2;
end;

endfunction
