function [tfr,t,f] = tfrstft(x,t,N,h,ptrace,opt_plot);
// Short time Fourier transform.
// Calling Sequence
//	[TFR,T,F]=tfrstft(X)
//	[TFR,T,F]=tfrstft(X,T)
//	[TFR,T,F]=tfrstft(X,T,N)
//	[TFR,T,F]=tfrstft(X,T,N,H
//	[TFR,T,F]=tfrstft(X,T,N,H,TRACE)
//	[TFR,T,F]=tfrstft(...,'plot')
//    Parameters
//	X     : signal.
//	T     : time instant(s)          (default : 1:length(X)).
//	N     : number of frequency bins (default : length(X)).
//	H     : frequency smoothing window, H being normalized so as to   be  of unit energy.      (default : Hamming(N/4)). 
//	TRACE : if nonzero, the progression of the algorithm is shown   (default : 0).
//      'plot':	if one input parameter is 'plot',  tfrstft runs tfrqview. and TFR will be plotted
//	TFR   : time-frequency decomposition (complex values). The frequency axis is graduated from -0.5 to 0.5.
//	F     : vector of normalized frequencies.
// Desciption
//    tfrstft computes the short-time Fourier transform of a discrete-time signal X. 
//  Examples 
//       sig=[fmconst(128,0.2);fmconst(128,0.4)]; [tfr,t,f]=tfrstft(sig);
//       subplot(211); grayplot(t,f,abs(tfr)');
//       subplot(212); grayplot(t,f,atan(imag(tfr),real(tfr))');
// Authors
//      H. Nahrstaedt - Aug 2010
//	F. Auger, May-August 1994, July 1995.
//	Copyright (c) 1996 by CNRS (France).

//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 [nargout,nargin]=argn(0);

in_par=['x','t','N','h','ptrace','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;


[xrow,xcol] = size(x);
if (nargin < 1),
 error('At least 1 parameter is required');
elseif (nargin <= 2),
 N=xrow;
end;

hlength=floor(N/4);
hlength=hlength+1-modulo(hlength,2);

if (nargin == 1),
 t=1:xrow; h = tftb_window(hlength); ptrace=0;
elseif (nargin == 2) | (nargin == 3),
 h = tftb_window(hlength); ptrace = 0;
elseif (nargin == 4),
 ptrace = 0;
end;

if (N<0),
 error('N must be greater than zero');
end;
[trow,tcol] = size(t);
if (xcol~=1),
 error('X must have one column');
elseif (trow~=1),
 error('T must only have one row'); 
elseif (2^nextpow2(N)~=N),
 printf('For a faster computation, N should be a power of two\n');
end; 

[hrow,hcol]=size(h); Lh=(hrow-1)/2; 
if (hcol~=1)|(modulo(hrow,2)==0),
 error('H must be a smoothing window with odd length');
end;

h=h/norm(h);

tfr= zeros (N,tcol) ;  
if ptrace, disp('Short-time Fourier transform'); end;
for icol=1:tcol,
 ti= t(icol); tau=-min([round(N/2)-1,Lh,ti-1]):min([round(N/2)-1,Lh,xrow-ti]);
 indices= modulo(N+tau,N)+1; 
 if ptrace, disprog(icol,tcol,10); end;
 tfr(indices,icol)=x(ti+tau,1).*conj(h(Lh+1+tau));
end;

  tfr=tftb_fft(tfr); 

if ptrace, printf('\n'); end;

if (plotting),
 tfrqview(abs(tfr).^2,x,t,'tfrstft',h);
end
if (nargout==3),
//  if modulo(N,2)==0, 
//   f=[0:N/2-1 -N/2:-1]'/N;
//  else
//   f=[0:(N-1)/2 -(N-1)/2:-1]'/N;  
//  end;
  f=(0:N-1)/N;
end;
endfunction

